function restart_index = crash_recovery(inputfile, temp_outputfile)
%
% CRASH_RECOVERY : If there is a failure during the running of 
% 'estimate_model', run this routine.  It reads the temp file 
% 'tempparams.out', which contains the set of parameter values at
% the current best solution to the ML optimization problem.  The
% parameters are then written in nicer form to the output file
% specified in the inputs to this routine.  The last loop the
% routine was in is returned in the argument.  It is used as the
% restart value.  
%
% Greg Duffee
% March 2006, revised September 2006
%

% hardwire the number of macro factors
n = 3; 

[macrodatafile yielddatafile maturitydatafile numbonds ...
 periodlength firstobs lastobs shortrate_index ...
 use_ols_start use_latent numloops numsimplex numderiv ...
 fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
 fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
 prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
 prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
 prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
 prem_macro_names prem_macro_flag ...
 prem_macro_initialval prem_macro_rand_stddev ...
 errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
] = read_input(inputfile, n); 

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

[old_freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

f_input_id = fopen('tempindex.out', 'r'); 
tempparams = textscan(f_input_id, '%f'); 
restart_index = tempparams{1};
fclose(f_input_id);

f_input_id = fopen('tempparams.out', 'r'); 
tempparams = textscan(f_input_id, '%f'); 
free_estimates = tempparams{1};
fclose(f_input_id);

all_estimates = poke_freeparams(free_estimates, allparams, free_index); 

[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
    = unpack_allparams(n, use_latent, all_estimates, numbonds); 
errorSD = abs(errorSD); 
% lower trangular sigma matrices do not necessarily have
% nonnegative values on their diagonals.  Here I make sure that
% this normalization is imposed.  
Sigma_f = zeros(n,n); 
index = 0;
for i = 1:n
  for j = 1:i
    index = index + 1;
    Sigma_f(i,j) = fund_S(index);
  end
end
Q = Sigma_f*Sigma_f';
[tmp p] = chol(Q); 
if (p == 0)
  Sigma_f = tmp'; 
  index = 0; 
  for i = 1:n
    for j = 1:i
      index = index + 1;
      fund_S(index) = Sigma_f(i,j);
    end
  end
end
% now for the term premia sigma matrix.  
if (use_latent == 0)
  prem_K = prem_K_initialval;
  prem_S = prem_S_initialval;
else
  Sigma_omega = zeros(n,n); 
  index = 0;
  for i = 1:n
    for j = 1:i
      index = index + 1;
      Sigma_omega(i,j) = prem_S(index);
    end
  end
  Q = Sigma_omega*Sigma_omega';
  [tmp p] = chol(Q); 
  if (p == 0) 
    Sigma_omega = tmp'; 
    index = 0; 
    for i = 1:n
      for j = 1:i
        index = index + 1;
        prem_S(index) = Sigma_omega(i,j);
      end
    end
  end
end

restrict_unconmean = 0; 
mineigen_allowed = -1e10; 
[funval funderiv loglikes loglike_derivs filtered_values] = ...
    kalman_only_loglike(free_estimates, ...
                        all_estimates, free_index, ...
                        macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, restrict_unconmean, ...
                        mineigen_allowed); 
score = -funderiv; 
loglike = -funval; 
exitflag = -99; 
robust_stderrs = zeros(length(free_estimates), 1); 
outerprod_stderrs = zeros(length(free_estimates), 1); 

write_output(inputfile, temp_outputfile, n, ...
             fund_K, fund_S, prem_K, ...
             prem_S, prem_c, prem_macro, errorSD, loglike, ...
             exitflag, score, robust_stderrs, outerprod_stderrs); 


return

