% Greg Duffee
% June 2007
% 
%

clear

%---------------------------------------------------------------
%
% User specifies the model to be studied.
%
%---------------------------------------------------------------

useLate = 0;  % must be, in order to study 1980.  
modelUsed = 1; % 1 == macro; 2 = latent; 3 = both;

%---------------------------------------------------------------
%
% Pull data file.  
%
%---------------------------------------------------------------

if (modelUsed==1)
  outputfile = char('..\matlabFiles\full_onlymacro2.out'); 
else
  if (modelUsed==2)
    outputfile = char('..\matlabFiles\full_onlyomega2.out'); 
  else
    outputfile = char('..\matlabFiles\full_both4.out'); 
  end
end

%---------------------------------------------------------------
%
% Read output file, set up parameters of the model.
%
%---------------------------------------------------------------

% hardwire the number of macro factors
n = 3; 

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 
[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 

macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;

[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 

[Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                mu_x_equiv, K_x_equiv, Sigma_x, ...
                                maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 
[uncon_mean uncon_var] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                     Sigma_f, K_omega, Sigma_omega); 
[errors loglikes filtered_values cond_vars ...
 contemp_vars tmp1 prev_cond_expectations] = ...
    kalman_recursion_fast([macrodata yielddata], mu_x, K_x, Sigma_x, ...
                     A, B, H, uncon_mean, uncon_var, 1e-20);

factorInnovations = filtered_values - prev_cond_expectations; 

%---------------------------------------------------------------
%
% Study in detail 1980Q3 and 1980Q4
%
%---------------------------------------------------------------

tminus1 = 78;             % 1980Q3 
dateT = tminus1 + 1;      % 1980Q4

yieldIdx = 5; % study 7-year bond yield.  

tminus1_actual = [macrodata(tminus1,:)'; yielddata(tminus1,:)'];
t_actual = [macrodata(dateT,:)'; yielddata(dateT,:)'];

oldFactors = filtered_values(tminus1,:)';
newFactors = filtered_values(dateT,:)';

fittedOldObs = A + B * oldFactors;
fittedNewObs = A + B * newFactors;
factorForecasts = prev_cond_expectations(dateT,:)';
allForecasts = A + B * factorForecasts;

factorShocks = factorInnovations(dateT,:)'; 
allShocks = B*factorShocks;

if use_latent
  fittedObscondMacro = A + B * [newFactors(1:3); forecasts(4:6)];
  fittedShockscondMacro = B * [factorShocks(1:3); zeros(3,1)];
else
  fittedObscondMacro = A + B * newFactors;
  fittedShockscondMacro = B * factorShocks;
end

% get constant-premia factor loading

[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      zeros(size(lambda_1f)), lambda0); 
[Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                mu_x_equiv, K_x_equiv, Sigma_x, ...
                                maturities, periodlength); 
[altA altB H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 

if use_latent 
  altFittedShockscondMacro = altB * [factorShocks(1:3); zeros(3,1)];
else
  altFittedShockscondMacro = altB * factorShocks;
end

%
% long-run forecast of inflation

myHorizon = 40;
ident = eye(size(K_x,1));
meansOverHorizon = uncon_mean + ...
    (1/myHorizon)*inv(ident - K_x)* ...
    (ident - K_x^myHorizon)*(oldFactors - uncon_mean); 

%-------------------------------------------------------------
%
% print out results
%
%-------------------------------------------------------------


fprintf(1, '\nAnalysis of 1980Q3 and 1980Q4\n');
fprintf(1,   '-----------------------------\n\n');

fprintf(1, 'Actual values\n'); 
fprintf(1, ['Quarter Inflation   Output g  short rate' ...
            ' 7-year yield (all in percent)\n']);
fprintf(1, '1980Q3 %10.3f %10.3f %10.3f %10.3f\n', ...
        100*tminus1_actual(1), 100*tminus1_actual(2), ...
        100*tminus1_actual(3), 100*tminus1_actual(8));
fprintf(1, '1980Q4 %10.3f %10.3f %10.3f %10.3f\n', ...
        100*t_actual(1), 100*t_actual(2), ...
        100*t_actual(3), 100*t_actual(8));

fprintf(1, '\n1980Q3 filtered values\n');
fprintf(1, ['Inflation     Output g  short rate' ...
            '   7-year yield (all in percent)\n']);
fprintf(1, '%10.3f %10.3f %10.3f %10.3f \n', ...
        100*fittedOldObs(1), 100*fittedOldObs(2), 100*fittedOldObs(3), ...
        100*fittedOldObs(8)); 
fprintf(1, '\n1980Q3 one-quarter-ahead forecasts\n');
fprintf(1, ['Inflation     Output g  short rate' ...
            '   7-year yield (all in percent)\n']);
fprintf(1, '%10.3f %10.3f %10.3f %10.3f \n', ...
        100*allForecasts(1), 100*allForecasts(2), 100*allForecasts(3), ...
        100*allForecasts(8)); 
fprintf(1, '\n1980Q4 filtered values\n');
fprintf(1, ['Inflation     Output g  short rate' ...
            '   7-year yield (all in percent)\n']);
fprintf(1, '%10.3f %10.3f %10.3f %10.3f \n', ...
        100*fittedNewObs(1), 100*fittedNewObs(2), 100*fittedNewObs(3), ...
        100*fittedNewObs(8)); 

fprintf(1, '\nShock decomposition for 7-year bond yield (percent)\n');
fprintf(1, 'Total shock:                   %10.3f\n', 100*allShocks(8));
fprintf(1, 'News about future short rates: %10.3f\n', ...
        100*altFittedShockscondMacro(8));
fprintf(1, 'Risk premia news related to macro:   %10.3f\n', ...
        100*(fittedShockscondMacro(8) - ...
             altFittedShockscondMacro(8)));
fprintf(1, 'Risk premia news unrelated to macro: %10.3f\n', ...
        100*(allShocks(8) - fittedShockscondMacro(8)));

fprintf(1, ['\nForecast as of 1980Q3 of average inflation rate over ' ...
            'next ten years.\n']);
fprintf(1, 'In percent:  %10.3f\n', 100*meansOverHorizon(1));
