% Greg Duffee
% June 2007
% 
% Display parameters of the chosen model.
%

clear

%---------------------------------------------------------------
%
% User specifies the model to be studied.
%
%---------------------------------------------------------------

useLate = 0; 
modelUsed = 2; % 1 == macro; 2 = latent; 3 = both;

%---------------------------------------------------------------
%
% Pull data file.  
%
%---------------------------------------------------------------

if useLate
  if (modelUsed==1)
  outputfile = char('..\matlabFiles\late_onlymacro2.out'); 
  else
    if (modelUsed==2)
      outputfile = char('..\matlabFiles\late_onlyomega3.out'); 
    else
      outputfile = char('..\matlabFiles\late_both2.out'); 
    end
  end
else
  if (modelUsed==1)
    outputfile = char('..\matlabFiles\full_onlymacro2.out'); 
  else
    if (modelUsed==2)
      outputfile = char('..\matlabFiles\full_onlyomega2.out'); 
    else
      outputfile = char('..\matlabFiles\full_both4.out'); 
    end
  end
end

%---------------------------------------------------------------
%
% Read output file, set up parameters of the model.
%
%---------------------------------------------------------------

% hardwire the number of macro factors
n = 3; 

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

% how many parameters are free?  
[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K, fund_S, prem_K, prem_S, ...
    prem_c, prem_macro, errorSD, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

%
% Pull off standard errors
%

fid = fopen(outputfile, 'r');
frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 22) & (tmp(1:23) == 'STANDARD ERRORS (Robust'))
        foundline = 1; 
    end
end
numlines = double(length(freeparams));
output = textscan(fid, '%d %f %f', numlines); 
OPstderrs = output{3}; 
fclose(fid); 

% 
% get log-like
%

mineigen_allowed = -1; 
restrict_unconmean = 1; 
funval = kalman_only_loglike(freeparams, ...
                        allparams, free_index, ...
                        macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, restrict_unconmean, ...
                        mineigen_allowed); 
loglike = -funval;

%
% put parameters into relevant matrices
%

[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;
[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 

%
% I do the same by hand for standard errors
%

K_f_SE = reshape(OPstderrs(1:9), 3, 3);
Sigma_f_SE = zeros(3,3);
Sigma_f_SE(1,1) = OPstderrs(10);
Sigma_f_SE(2,1) = OPstderrs(11);
Sigma_f_SE(2,2) = OPstderrs(12);
Sigma_f_SE(3,1) = OPstderrs(13);
Sigma_f_SE(3,2) = OPstderrs(14);
Sigma_f_SE(3,3) = OPstderrs(15);

if (modelUsed == 1) 
  K_omega_SE = reshape(OPstderrs(16:24), 3, 3);
  Sigma_omega_SE = zeros(3,3);
  Sigma_omega_SE(1,1) = OPstderrs(25);
  Sigma_omega_SE(2,1) = OPstderrs(26);
  Sigma_omega_SE(2,2) = OPstderrs(27);
  Sigma_omega_SE(3,1) = OPstderrs(28);
  Sigma_omega_SE(3,2) = OPstderrs(29);
  Sigma_omega_SE(3,3) = OPstderrs(30);
  lambda0_SE = OPstderrs(16:18);
  lambda_1f_SE = reshape(OPstderrs(19:27),3,3);
  errorSD_SE = OPstderrs(28:30);
end

if (modelUsed == 2) 
  K_omega_SE = reshape(OPstderrs(16:24), 3, 3);
  Sigma_omega_SE = zeros(3,3);
  Sigma_omega_SE(1,1) = OPstderrs(25);
  Sigma_omega_SE(2,1) = OPstderrs(26);
  Sigma_omega_SE(2,2) = OPstderrs(27);
  Sigma_omega_SE(3,1) = OPstderrs(28);
  Sigma_omega_SE(3,2) = OPstderrs(29);
  Sigma_omega_SE(3,3) = OPstderrs(30);
  lambda0_SE = OPstderrs(31:33);
  errorSD_SE = OPstderrs(34:36);
end

if (modelUsed == 3) 
  K_omega_SE = reshape(OPstderrs(16:24), 3, 3);
  Sigma_omega_SE = zeros(3,3);
  Sigma_omega_SE(1,1) = OPstderrs(25);
  Sigma_omega_SE(2,1) = OPstderrs(26);
  Sigma_omega_SE(2,2) = OPstderrs(27);
  Sigma_omega_SE(3,1) = OPstderrs(28);
  Sigma_omega_SE(3,2) = OPstderrs(29);
  Sigma_omega_SE(3,3) = OPstderrs(30);
  lambda0_SE = OPstderrs(31:33);
  lambda_1f_SE = reshape(OPstderrs(34:42),3,3);
  errorSD_SE = OPstderrs(43:45);
end
  
sigmaScale = 1000; 
sSigma_f = sigmaScale*Sigma_f;
sSigma_f_SE = sigmaScale*Sigma_f_SE;
sSigma_omega = sigmaScale*Sigma_omega;
sSigma_omega_SE = sigmaScale*Sigma_omega_SE;

%---------------------------------------------------------------
%
% print out parameters in form suitable for tables
%
%---------------------------------------------------------------

if useLate 
  fprintf(1, 'Parameter estimates based on later sample.\n');
else
  fprintf(1, 'Parameter estimates based on full sample.\n');
end
if (modelUsed == 1) 
  fprintf(1, 'Model is standard macro-finance model.\n');
end
if (modelUsed == 2) 
  fprintf(1, 'Model is general null.\n');
end
if (modelUsed == 3) 
  fprintf(1, 'Model is alternative hypothesis model.\n');
end
fprintf(1, '\n---------------------------------------------------\n\n');
fprintf(1, 'Number of free parameters = %d. \n', length(freeparams));
fprintf(1, 'Log likelihood = %12.4f\n\n', loglike);

fprintf(1, 'Unconditional mean of macro variables + 100*SE of noise\n');
fprintf(1, 'Inflation:      %12.4f %12.3f\n', macroMean(1), errorSD(1));
fprintf(1, '                               (%8.3f)\n', errorSD_SE(1));
fprintf(1, 'Output growth:  %12.4f %12.3f\n', macroMean(2), errorSD(2));
fprintf(1, '                               (%8.3f)\n', errorSD_SE(2));
fprintf(1, 'Short rate:     %12.4f %12.3f\n', macroMean(3), 0);
fprintf(1, '                               ( --- )\n');

fprintf(1, '\nK_f\n\n');
fprintf(1, '    pi   delta g    r\n');
fprintf(1, 'pi   %10.3f %10.3f %10.3f\n', ...
        K_f(1,1), K_f(1,2), K_f(1,3));
fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
        K_f_SE(1,1), K_f_SE(1,2), K_f_SE(1,3));
fprintf(1, 'd g  %10.3f %10.3f %10.3f\n', ...
        K_f(2,1), K_f(2,2), K_f(2,3));
fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
        K_f_SE(2,1), K_f_SE(2,2), K_f_SE(2,3));
fprintf(1, '  r  %10.3f %10.3f %10.3f\n', ...
        K_f(3,1), K_f(3,2), K_f(3,3));
fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
        K_f_SE(3,1), K_f_SE(3,2), K_f_SE(3,3));
fprintf(1, '\n1000*Sigma_f\n\n');
fprintf(1, '    pi   delta g    r\n');
fprintf(1, 'pi   %10.3f %10.3f %10.3f\n', ...
        sSigma_f(1,1), sSigma_f(1,2), sSigma_f(1,3));
fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
        sSigma_f_SE(1,1), sSigma_f_SE(1,2), sSigma_f_SE(1,3));
fprintf(1, 'd g  %10.3f %10.3f %10.3f\n', ...
        sSigma_f(2,1), sSigma_f(2,2), sSigma_f(2,3));
fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
        sSigma_f_SE(2,1), sSigma_f_SE(2,2), sSigma_f_SE(2,3));
fprintf(1, '  r  %10.3f %10.3f %10.3f\n', ...
        sSigma_f(3,1), sSigma_f(3,2), sSigma_f(3,3));
fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
        sSigma_f_SE(3,1), sSigma_f_SE(3,2), sSigma_f_SE(3,3));

fprintf(1, '\nLambda0\n');
fprintf(1, 'pi %10.3f\n', lambda0(1));
fprintf(1, '  (%8.3f)\n', lambda0_SE(1));
fprintf(1, 'pi %10.3f\n', lambda0(2));
fprintf(1, '  (%8.3f)\n', lambda0_SE(2));
fprintf(1, 'pi %10.3f\n', lambda0(3));
fprintf(1, '  (%8.3f)\n', lambda0_SE(3));

if (modelUsed ~= 2) 
  fprintf(1, '\nLambda1_f\n\n');
  fprintf(1, '    pi   delta g    r\n');
  fprintf(1, 'pi   %10.3f %10.3f %10.3f\n', ...
          lambda_1f(1,1), lambda_1f(1,2), lambda_1f(1,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          lambda_1f_SE(1,1), lambda_1f_SE(1,2), lambda_1f_SE(1,3));
  fprintf(1, 'd g  %10.3f %10.3f %10.3f\n', ...
          lambda_1f(2,1), lambda_1f(2,2), lambda_1f(2,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          lambda_1f_SE(2,1), lambda_1f_SE(2,2), lambda_1f_SE(2,3));
  fprintf(1, '  r  %10.3f %10.3f %10.3f\n', ...
          lambda_1f(3,1), lambda_1f(3,2), lambda_1f(3,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          lambda_1f_SE(3,1), lambda_1f_SE(3,2), lambda_1f_SE(3,3));
end


if (modelUsed > 1) 
  fprintf(1, '\nK_omega\n\n');
  fprintf(1, '    pi   delta g    r\n');
  fprintf(1, 'pi   %10.3f %10.3f %10.3f\n', ...
          K_omega(1,1), K_omega(1,2), K_omega(1,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          K_omega_SE(1,1), K_omega_SE(1,2), K_omega_SE(1,3));
  fprintf(1, 'd g  %10.3f %10.3f %10.3f\n', ...
          K_omega(2,1), K_omega(2,2), K_omega(2,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          K_omega_SE(2,1), K_omega_SE(2,2), K_omega_SE(2,3));
  fprintf(1, '  r  %10.3f %10.3f %10.3f\n', ...
          K_omega(3,1), K_omega(3,2), K_omega(3,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          K_omega_SE(3,1), K_omega_SE(3,2), K_omega_SE(3,3));
  fprintf(1, '\n1000*Sigma_omega\n\n');
  fprintf(1, '    pi   delta g    r\n');
  fprintf(1, 'pi   %10.3f %10.3f %10.3f\n', ...
          sSigma_omega(1,1), sSigma_omega(1,2), sSigma_omega(1,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          sSigma_omega_SE(1,1), sSigma_omega_SE(1,2), sSigma_omega_SE(1,3));
  fprintf(1, 'd g  %10.3f %10.3f %10.3f\n', ...
          sSigma_omega(2,1), sSigma_omega(2,2), sSigma_omega(2,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          sSigma_omega_SE(2,1), sSigma_omega_SE(2,2), sSigma_omega_SE(2,3));
  fprintf(1, '  r  %10.3f %10.3f %10.3f\n', ...
          sSigma_omega(3,1), sSigma_omega(3,2), sSigma_omega(3,3));
  fprintf(1, '     (%8.3f) (%8.3f) (%8.3f)\n', ...
          sSigma_omega_SE(3,1), sSigma_omega_SE(3,2), sSigma_omega_SE(3,3));
end
