% Greg Duffee
% June 2007
% 
% Display performance metrics.
%

clear

useLate = 1; 

n = 3; 

%---------------------------------------------------------------
%
% Get the name of the standard model output file.  Then read output
% file, setup parameters, get forecast errors. 
%
%---------------------------------------------------------------

if useLate 
  outputfile = char('..\matlabFiles\late_onlymacro2.out'); 
else
  outputfile = char('..\matlabFiles\full_onlymacro.out'); 
end

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

use_ols_start = 0; 

[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

[nullFunval loglikes filtered_values] = ...
    kalman_only_loglike_noderiv(freeparams, ...
                        allparams, free_index, macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, 1e-8, 0, -1);    

paramvec = poke_freeparams(freeparams, allparams, free_index); 
[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, paramvec, numbonds); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;
[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 

[Avec Bmat B_pricing_mat] = ...
    get_yield_mapping(shortrate_index, ...
                      mu_x_equiv, K_x_equiv, Sigma_x, ...
                      maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 
[uncon_mean uncon_vcv] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                     Sigma_f, K_omega, Sigma_omega); 
[errors restrLoglikes restrFilteredValues cond_vars ...
 contemp_vars observed_stepahead_vars] = ...
    kalman_recursion_fast( ...
        [macrodata yielddata], mu_x, K_x, Sigma_x, ...
        A, B, H, uncon_mean, uncon_vcv, 1e-20);


restrFilteredYields = repmat(Avec', size(yielddata,1),1) + ...
    restrFilteredValues * Bmat'; 
restrFilteredMacro = restrFilteredValues(:,1:3);
restrContempErrors = [(macrodata - restrFilteredMacro) ...
                    yielddata - restrFilteredYields];
restrStdContempErrors = 100*std(restrContempErrors)'; 

RMSE = 100*sqrt(mean(errors .* errors)');
RMSE_ex1 = 100*sqrt(mean(errors(2:end,:) .* errors(2:end,:))');

restrSummary = [restrStdContempErrors errors(1,:)' RMSE RMSE_ex1];

%---------------------------------------------------------------
%
% Get the name of the general null output file.  Then read output
% file, setup parameters, get forecast errors. 
%
%---------------------------------------------------------------

if useLate 
  outputfile = char('..\matlabFiles\late_onlyomega3.out'); 
else
  outputfile = char('..\matlabFiles\full_onlyomega2.out'); 
end

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

use_ols_start = 0; 

[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

[nullFunval loglikes filtered_values] = ...
    kalman_only_loglike_noderiv(freeparams, ...
                        allparams, free_index, macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, 1e-8, 0, -1);    

paramvec = poke_freeparams(freeparams, allparams, free_index); 
[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, paramvec, numbonds); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;
[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 

[Avec Bmat B_pricing_mat] = ...
    get_yield_mapping(shortrate_index, ...
                      mu_x_equiv, K_x_equiv, Sigma_x, ...
                      maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 
[uncon_mean uncon_vcv] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                     Sigma_f, K_omega, Sigma_omega); 
[errors nullLoglikes nullFilteredValues cond_vars ...
 contemp_vars observed_stepahead_vars ...
 prev_cond_expectations] = ...
    kalman_recursion_fast( ...
        [macrodata yielddata], mu_x, K_x, Sigma_x, ...
        A, B, H, uncon_mean, uncon_vcv, 1e-20);

nullFilteredYields = repmat(Avec', size(yielddata,1),1) + ...
    nullFilteredValues * Bmat'; 
nullFilteredMacro = nullFilteredValues(:,1:3);
nullContempErrors = [(macrodata - nullFilteredMacro) ...
                    yielddata - nullFilteredYields];
nullStdContempErrors = 100*std(nullContempErrors)'; 

RMSE = 100*sqrt(mean(errors .* errors)');
RMSE_ex1 = 100*sqrt(mean(errors(2:end,:) .* errors(2:end,:))');

nullSummary = [nullStdContempErrors errors(1,:)' RMSE RMSE_ex1];

%---------------------------------------------------------------
%
% Get the name of the alternative output file.  Then read output
% file, setup parameters, get forecast errors. 
%
%---------------------------------------------------------------


if useLate 
  outputfile = char('..\matlabFiles\late_both2.out'); 
else
  outputfile = char('..\matlabFiles\full_both4.out'); 
end

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 

use_ols_start = 0; 

[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

[altFunval loglikes filtered_values] = ...
    kalman_only_loglike_noderiv(freeparams, ...
                        allparams, free_index, macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, 1e-8, 1, -1);    

paramvec = poke_freeparams(freeparams, allparams, free_index); 
[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, paramvec, numbonds); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
mu_f = (eye(n) - K_f) * macroMean;
[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 


[Avec Bmat B_pricing_mat] = ...
    get_yield_mapping(shortrate_index, ...
                      mu_x_equiv, K_x_equiv, Sigma_x, ...
                      maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 
[uncon_mean uncon_vcv] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                     Sigma_f, K_omega, Sigma_omega); 
[errors altLoglikes altFilteredValues cond_vars ...
 contemp_vars observed_stepahead_vars ...
 prev_cond_expectations] = ...
    kalman_recursion_fast( ...
        [macrodata yielddata], mu_x, K_x, Sigma_x, ...
        A, B, H, uncon_mean, uncon_vcv, 1e-10);

altFilteredYields = repmat(Avec', size(yielddata,1),1) + ...
    altFilteredValues * Bmat'; 
altFilteredMacro = altFilteredValues(:,1:3);
altContempErrors = [(macrodata - altFilteredMacro) ...
                    yielddata - altFilteredYields];
altStdContempErrors = 100*std(altContempErrors)'; 

RMSE = 100*sqrt(mean(errors .* errors)');
RMSE_ex1 = 100*sqrt(mean(errors(2:end,:) .* errors(2:end,:))');

altSummary = [altStdContempErrors errors(1,:)' RMSE RMSE_ex1]; 

%---------------------------------------------------------------
%
% print out values in form suitable for tables
%
%---------------------------------------------------------------

if useLate 
  fprintf(1, '\nValues based on later sample.\n');
else
  fprintf(1, '\nValues based on full sample.\n');
end

fprintf(1,'Standard deviation of measurement error.  These are\n');
fprintf(1,'sample standard deviations.\n');

fprintf(1,'                      Standard    General null   Alt model\n');
fprintf(1,'Inflation (percent)  %10.2f %10.2f %10.2f\n', ...
        restrSummary(1,1), nullSummary(1,1), altSummary(1,1));
fprintf(1,'output gr (percent)  %10.2f %10.2f %10.2f\n', ...
        restrSummary(2,1), nullSummary(2,1), altSummary(2,1));
fprintf(1,'7yr yield (bp)       %10.2f %10.2f %10.2f\n\n', ...
        100*restrSummary(8,1), 100*nullSummary(8,1), 100*altSummary(8,1));

fprintf(1,'Root mean squared error of one-quarter-ahead forecasts\n');
fprintf(1,'These are all in percent.\n');
fprintf(1,'              Standard   General null    Alt model\n');
fprintf(1,'Inflation %10.2f %10.2f %10.2f\n', ...
        restrSummary(1,3), nullSummary(1,3), altSummary(1,3));
fprintf(1,'output gr %10.2f %10.2f %10.2f\n', ...
        restrSummary(2,3), nullSummary(2,3), altSummary(2,3));
fprintf(1,'Short r   %10.2f %10.2f %10.2f\n', ...
        restrSummary(3,3), nullSummary(3,3), altSummary(3,3));
fprintf(1,'7yr yield %10.2f %10.2f %10.2f\n\n', ...
        restrSummary(8,3), nullSummary(8,3), altSummary(8,3));


fprintf(1,'Root mean squared error of one-quarter-ahead forecasts\n');
fprintf(1,'These are all in percent.\n');
fprintf(1,'The first observation is excluded.\n'); 
fprintf(1,'              Standard   General null    Alt model\n');
fprintf(1,'Inflation %10.2f %10.2f %10.2f\n', ...
        restrSummary(1,4), nullSummary(1,4), altSummary(1,4));
fprintf(1,'output gr %10.2f %10.2f %10.2f\n', ...
        restrSummary(2,4), nullSummary(2,4), altSummary(2,4));
fprintf(1,'Short r   %10.2f %10.2f %10.2f\n', ...
        restrSummary(3,4), nullSummary(3,4), altSummary(3,4));
fprintf(1,'7yr yield %10.2f %10.2f %10.2f\n\n', ...
        restrSummary(8,4), nullSummary(8,4), altSummary(8,4));


