% Greg Duffee
% June 2007
% 
% Display properties of models.
%

clear

useLate = 0; 

n = 3; 

%---------------------------------------------------------------
%
% Get the name of the standard model output file.  Then read output
% file, setup parameters, get forecast errors. 
%
%---------------------------------------------------------------

if useLate 
  outputfile = char('..\matlabFiles\late_onlymacro2.out'); 
else
  outputfile = char('..\matlabFiles\full_onlymacro.out'); 
end

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

use_ols_start = 0; 

[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

[nullFunval loglikes filtered_values] = ...
    kalman_only_loglike_noderiv(freeparams, ...
                        allparams, free_index, macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, 1e-8, 0, -1);    

paramvec = poke_freeparams(freeparams, allparams, free_index); 
[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, paramvec, numbonds); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;

[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 
[uncon_mean uncon_var] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                     Sigma_f, K_omega, Sigma_omega); 
[Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                mu_x_equiv, K_x_equiv, Sigma_x, ...
                                maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 

[errors restrLoglikes filteredValues] = ...
    kalman_recursion_fast( ...
        [macrodata yielddata], mu_x, K_x, Sigma_x, ...
        A, B, H, uncon_mean, uncon_var, 1e-20);

[restrMeanExcessRets restrStdExcessRets restrStdCondMean ...
 restrStdInnov restrMacroComponent restrCovarComponent ...
 restrResidComponent restrAr1] = ...
    table5_newcode(mu_f, K_f, Sigma_f, K_omega, Sigma_omega, lambda0, ...
                   lambda_1f, use_latent, maturities, periodlength, ...
                   shortrate_index, filteredValues); 

%---------------------------------------------------------------
%
% Get the name of the general null output file.  Then read output
% file, setup parameters, get forecast errors. 
%
%---------------------------------------------------------------

if useLate 
  outputfile = char('..\matlabFiles\late_onlyomega3.out'); 
else
  outputfile = char('..\matlabFiles\full_onlyomega2.out'); 
end

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

use_ols_start = 0; 

[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

[nullFunval loglikes filtered_values] = ...
    kalman_only_loglike_noderiv(freeparams, ...
                        allparams, free_index, macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, 1e-8, 0, -1);    

paramvec = poke_freeparams(freeparams, allparams, free_index); 
[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, paramvec, numbonds); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;
[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 
[uncon_mean uncon_var] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                     Sigma_f, K_omega, Sigma_omega); 
[Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                mu_x_equiv, K_x_equiv, Sigma_x, ...
                                maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 

[errors nullLoglikes filteredValues] = ...
    kalman_recursion_fast( ...
        [macrodata yielddata], mu_x, K_x, Sigma_x, ...
        A, B, H, uncon_mean, uncon_var, 1e-20);

[nullMeanExcessRets nullStdExcessRets nullStdCondMean ...
 nullStdInnov nullMacroComponent nullCovarComponent ...
 nullResidComponent nullAr1] = ...
    table5_newcode(mu_f, K_f, Sigma_f, K_omega, Sigma_omega, lambda0, ...
                   lambda_1f, use_latent, maturities, periodlength, ...
                   shortrate_index, filteredValues); 

%---------------------------------------------------------------
%
% Get the name of the alternative output file.  Then read output
% file, setup parameters, get forecast errors. 
%
%---------------------------------------------------------------


if useLate 
  outputfile = char('..\matlabFiles\late_both2.out'); 
else
  outputfile = char('..\matlabFiles\full_both4.out'); 
end

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
        errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
       ] = read_input(outputfile, n); 

use_ols_start = 0; 

[freeparams allparams free_index] = setup_initparams( ...
    use_latent, fund_K_initialval, ...
    fund_S_initialval, prem_K_initialval, prem_S_initialval, ...
    prem_c_initialval, prem_macro_initialval, errorSD_initialval, ...
    fund_K_flag, fund_S_flag, ...
    prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
    errorSD_flag, macrodata, use_ols_start); 

[altFunval loglikes filtered_values] = ...
    kalman_only_loglike_noderiv(freeparams, ...
                        allparams, free_index, macrodata, yielddata, ...
                        maturities, periodlength, use_latent, ...
                        shortrate_index, 1e-8, 1, -1);    

paramvec = poke_freeparams(freeparams, allparams, free_index); 
[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, paramvec, numbonds); 
[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
mu_f = (eye(n) - K_f) * macroMean;
[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 

[uncon_mean uncon_var] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, ...
                     Sigma_f, K_omega, Sigma_omega); 
[Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                mu_x_equiv, K_x_equiv, Sigma_x, ...
                                maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 

[errors altLoglikes filteredValues] = ...
    kalman_recursion_fast( ...
        [macrodata yielddata], mu_x, K_x, Sigma_x, ...
        A, B, H, uncon_mean, uncon_var, 1e-20);

[altMeanExcessRets altStdExcessRets altStdCondMean ...
 altStdInnov altMacroComponent altCovarComponent ...
 altResidComponent altAr1] = ...
    table5_newcode(mu_f, K_f, Sigma_f, K_omega, Sigma_omega, lambda0, ...
                   lambda_1f, use_latent, maturities, periodlength, ...
                   shortrate_index, filteredValues); 

%---------------------------------------------------------------
%
% print out values in form suitable for tables
%
%---------------------------------------------------------------

if useLate 
  fprintf(1, '\nValues based on later sample.\n');
else
  fprintf(1, '\nValues based on full sample.\n');
end

fprintf(1, '\nMean excess log returns (percent)\n');
fprintf(1, 'Model           2-yr    7-yr\n');
fprintf(1, '   Std   %9.2f %9.2f\n', 100*restrMeanExcessRets(2), ...
        100*restrMeanExcessRets(5));
fprintf(1, '   Null  %9.2f %9.2f\n', 100*nullMeanExcessRets(2), ...
        100*nullMeanExcessRets(5));
fprintf(1, '   Alt   %9.2f %9.2f\n', 100*altMeanExcessRets(2), ...
        100*altMeanExcessRets(5));

fprintf(1, '\nStd dev of excess log returns (percent)\n');
fprintf(1, 'Model           2-yr    7-yr\n');
fprintf(1, '   Std   %9.2f %9.2f\n', 100*restrStdExcessRets(2), ...
        100*restrStdExcessRets(5));
fprintf(1, '   Null  %9.2f %9.2f\n', 100*nullStdExcessRets(2), ...
        100*nullStdExcessRets(5));
fprintf(1, '   Alt   %9.2f %9.2f\n', 100*altStdExcessRets(2), ...
        100*altStdExcessRets(5));

restrRatio = (restrStdCondMean' .^2) ./ (restrStdExcessRets .^2); 
nullRatio = (nullStdCondMean' .^2) ./ (nullStdExcessRets .^2); 
altRatio = (altStdCondMean' .^2) ./ (altStdExcessRets .^2); 

fprintf(1, '\nFraction of var due to var of cond mean\n');
fprintf(1, 'Model           2-yr    7-yr\n');
fprintf(1, '   Std   %9.2f %9.2f\n', ...
        restrRatio(2), restrRatio(5));
fprintf(1, '   Null  %9.2f %9.2f\n', ...
        nullRatio(2), nullRatio(5)); 
fprintf(1, '   Alt   %9.2f %9.2f\n', ...
        altRatio(2), altRatio(5)); 

restrRatio = diag(restrMacroComponent)' ./ (restrStdCondMean' .^2);
nullRatio = diag(nullMacroComponent)' ./ (nullStdCondMean' .^2);
altRatio = diag(altMacroComponent)' ./ (altStdCondMean' .^2);

fprintf(1, '\nFraction of cond mean var due to macro\n');
fprintf(1, 'Model           2-yr    7-yr\n');
fprintf(1, '   Std   %9.2f %9.2f\n', ...
        restrRatio(2), restrRatio(5));
fprintf(1, '   Null  %9.2f %9.2f\n', ...
        nullRatio(2), nullRatio(5)); 
fprintf(1, '   Alt   %9.2f %9.2f\n', ...
        altRatio(2), altRatio(5)); 

fprintf(1, '\nAR(1) coef for expected excess return\n');
fprintf(1, 'Model           2-yr    7-yr\n');
fprintf(1, '   Std   %9.2f %9.2f\n', ...
        restrAr1(2), restrAr1(5));
fprintf(1, '   Null  %9.2f %9.2f\n', ...
        nullAr1(2), nullAr1(5)); 
fprintf(1, '   Alt   %9.2f %9.2f\n', ...
        altAr1(2), altAr1(5)); 

