function [mu K vcv cholesky vector_form mu_companion K_companion resids] = ...
                         get_varparams(data, p)
% GET_VARPARAMS returns the parameters of OLS estimation of a VAR(p) given
% the matrix of data 'data'.  The parameters are returned in two different
% formats:  standard and first-order companion form.  The covariance matrix
% of the residuals is returned both as a matrix, 'vcv', and as its 
% Cholesky factorization, vcv = C*transpose(C).  Since Matlab uses the
% transpose of this factorization, I transpose the Matlab version.  The
% vector 'vector_form' stacks the standard parameters (and Cholesky 
% factorization) in a single vector.  

%
% Greg Duffee
% August 2005
%

numvars = size(data, 2); 
% make a big matrix that includes all of the lags. 
big_matrix = makecurrent(data, p); 

cur_data = big_matrix(2:end,:);
xmat = [ones((size(big_matrix,1)-1),1) big_matrix(1:(end-1),:)]; 

K = [];
mu = []; 
resids = [];
for i = 1:numvars
    [b bint r] = regress(cur_data(:,i), xmat); 
    K = [K; transpose(b(2:end))]; 
    resids = [resids r];
    mu = [mu; b(1)]; 
end
vcv = (1/size(resids,1)) * transpose(resids)*resids;
cholesky = transpose(chol(vcv)); 
chol_params = [];
for i = 1:size(cholesky,1)
     for j = 1:i
         chol_params = [chol_params; cholesky(i,j)]; 
     end
end

vector_form = [mu; reshape(transpose(K), [],1); chol_params];

K_companion = K; 
mu_companion = mu; 
if (p > 1) 
     mu_companion = [mu_companion; zeros(numvars*(p-1),1)];
     K_companion = ...
         [K_companion; eye((p-1)*numvars) zeros((p-1)*numvars, numvars)];
end

end
