function [Avec Bmat B_pricing_mat Aderivs Bderivs] = ...
    get_yield_mapping(shortrate_index, ...
                      mu_x_equiv, K_x_equiv, Sigma_x, ...
                      maturities, time_step)
%
% GET_YIELD_MAPPING calculates the affine mapping from the state vector
% to the bond yields.  The mapping is
%
% yields_t = Avec + Bmat * state_t
% 
% where yields_t and Avec are vectors.  The ordering is the ordering
% of the vector 'maturities'.  The short rate must be in position
% 'shortrate_index' of the state vector.  
%
% The function also returns derivatives of the A and B functions
% with respect to the following parameters:
% 
%

%
% Greg Duffee
% August 2006
%

n = length(mu_x_equiv); 
m = length(maturities); 

delta_x = zeros(n, 1);
delta_x(shortrate_index) = 1; 

Avec = zeros(m, 1); 
Bmat = zeros(m, n);
B_pricing_mat = zeros(m, n);

if (nargout < 4) 
  for i = 1:m
    this_mat = maturities(i); 
    this_a = Afunction(time_step, delta_x, mu_x_equiv, ...
                       K_x_equiv, Sigma_x, this_mat); 
    this_Bprime = Bfunction(delta_x, K_x_equiv, this_mat); 
    Avec(i) = this_a; 
    Bmat(i,:) = this_Bprime; 
    B_pricing_mat(i,:) = -time_step*this_mat*this_Bprime;
  end
else  
  Aderivs = zeros(m, (2*n + n^2 + n*(n+1)/2));
  Bderivs = zeros(m, n, (n+n^2)); 
  for i = 1:m
    this_mat = maturities(i); 
    [this_a this_a_deriv] = ...
        Afunction(time_step, delta_x, mu_x_equiv, ...
                  K_x_equiv, Sigma_x, this_mat);
    [this_Bprime this_b_deriv] = ...
        Bfunction(delta_x, K_x_equiv, this_mat);
    Avec(i) = this_a;
    Bmat(i,:) = this_Bprime;
    B_pricing_mat(i,:) = -time_step*this_mat*this_Bprime;
    Aderivs(i,:) = this_a_deriv;
    Bderivs(i, :, :) = this_b_deriv;
  end
end

return

