function [uncon_mean uncon_vcv mean_derivs vcv_derivs] = ...
    initial_forecast(use_latent, numbonds, mu_f, K_f, Sigma_f, K_omega, ...
                     Sigma_omega)
%
% Parameter order with latent factors, and their position in the vector
% vec(K_f)          (1 to n^2)
% vech(Sigma_f)     (n^2+1 to n^2+(n(n+1)/2))
% vec(K_omega)      (n^2+(n(n+1)/2)+1 to 2n^2+(n(n+1)/2))
% vech(Sigma_omega) (2n^2+(n(n+1)/2)+1 to 2+3n^2)
% lambda_0f         (n+3n^2+1 to 2n+3n^2)
% vec(lambda_1f)    (2n+3n^2+1 to 2n+4n^2)
% errorSD           (2n+4n^2+1 to 3n+4n^2+1)
%
% Parameter order without latent factors, and their position in the vector
% vec(K_f)          (1 to n^2)
% vech(Sigma_f)     (n^2+1 to n^2+(n(n+1)/2))
% lambda_0f         (n^2+(n(n+1)/2)+1 to n+n^2+(n(n+1)/2))
% vec(lambda_1f)    (n+n^2+(n(n+1)/2)+1 to n+2n^2+(n(n+1)/2))
% errorSD           (n+2n^2+(n(n+1)/2)+1 to 2n+2n^2+(n(n+1)/2) + 1)
%
% Also note the parameter vector of 'uncon_moments'.
%
% mu
% vec(K) 
% vech(Sigma) [row-by-row, not column-by-column].
%
%
% Greg Duffee
% September 2006
%

n = length(mu_f); 
if (use_latent == 1) 
  numparams = 3*n + 4*n^2 + + 1;
  dim = 2*n; 
else
  numparams = 2*n + 2*n^2 + (n*(n+1)/2) + 1; 
  dim = n; 
end

uncon_mean = zeros(dim,1); 
uncon_vcv = zeros(dim, dim); 

if (nargout > 2) 
  mean_derivs = zeros(dim, numparams); 
  vcv_derivs = zeros(dim, dim, numparams); 
  [fund_mean fund_vcv fund_stationary fund_mean_derivs ...
   fund_vcv_derivs] = uncon_moments(mu_f, K_f, Sigma_f); 
  num_fund_params = size(fund_mean_derivs,2); 
  uncon_mean(1:n) = fund_mean; 
  uncon_vcv(1:n,1:n) = fund_vcv; 
% here, mean is fixed.
%  mean_derivs(1:n,1:num_fund_params) = fund_mean_derivs; 
  vcv_derivs(1:n,1:n,1:(num_fund_params-n)) = ...
      fund_vcv_derivs(:,:,(n+1):end); 
  if (use_latent == 1) 
    [prem_mean prem_vcv prem_stationary prem_mean_derivs ...
     prem_vcv_derivs] = uncon_moments(zeros(n,1), K_omega, ...
                                      Sigma_omega);
    uncon_vcv((n+1):(2*n),(n+1):(2*n)) = prem_vcv; 
    start_point = n^2+(n*(n+1)/2) + 1; 
    end_point = 2*(n^2)+ (n*(n+1)); 
    vcv_derivs((n+1):(2*n),(n+1):(2*n),start_point:end_point) = ...
        prem_vcv_derivs(:,:,(n+1):end); 
  end
else
  [fund_mean fund_vcv] = uncon_moments(mu_f, K_f, Sigma_f); 
  uncon_mean(1:n) = fund_mean; 
  uncon_vcv(1:n,1:n) = fund_vcv; 
  if (use_latent == 1) 
    [prem_mean prem_vcv] = uncon_moments(zeros(n,1), K_omega, ...
                                         Sigma_omega);
    uncon_vcv((n+1):(2*n),(n+1):(2*n)) = prem_vcv; 
  end
end

return
