function [errors loglikes loglike_derivs ...
          filtered_values cond_vars contemp_vars] = ...
    kalman_recursion(datamatrix, mu, K, Sigma, A, B, H, ...
                     uncon_mean, uncon_vcv, ...
                     mu_derivs, K_derivs, vcv_derivs, ...
                     A_derivs, B_derivs, H_derivs, ... 
                     uncon_mean_derivs, uncon_vcv_derivs)
% KALMAN_RECURSION constructs the entire Kalman recursion.

%
% Greg Duffee
% March 2005, revised October 2005, Jan 2006, June 2006, August
% 2006, September 2006
%

[T d] = size(datamatrix); 
numfactors = length(mu); 
numparams = size(A_derivs,2); 

%
% loop through all observations.
%
errors = zeros(T, d); 
loglikes = zeros(T, 1); 
filtered_values = zeros(T, numfactors); 
cond_vars = zeros(T, (numfactors*(numfactors+1)/2));
contemp_vars = zeros(T, (numfactors*(numfactors+1)/2)); 
loglike_derivs = zeros(T, numparams); 
  
Q = Sigma*Sigma'; 
cond_expect = uncon_mean;
cond_var = uncon_vcv; 
partial_cond_expect = uncon_mean_derivs;
partial_cond_var = uncon_vcv_derivs; 

for index = 1:T
  observed_data = datamatrix(index,:)';
  [error loglike contemp_expect contemp_var ...
   forecast_expect forecast_var partial_loglike ...
   partial_forecast_expect partial_forecast_var] = ...
      single_recursion(observed_data, cond_expect, cond_var, ...
                       A, B, H, mu, K, Q, ...
                       partial_cond_expect, partial_cond_var, ...
                       A_derivs, B_derivs, H_derivs, mu_derivs, ...
                       K_derivs, vcv_derivs);
  loglikes(index) = loglike;
  errors(index,:) = error';
  filtered_values(index,:) = contemp_expect';
  cond_expect = forecast_expect;
  cond_var = forecast_var; 
  partial_cond_expect = partial_forecast_expect;
  partial_cond_var = partial_forecast_var; 
  cond_vars(index,:) = vech(cond_var)';
  contemp_vars(index,:) = vech(contemp_var)';
  loglike_derivs(index,:) = partial_loglike'; 
end

return


