function [errors loglikes ...
          filtered_values cond_vars contemp_vars ...
         observed_stepahead_vars, ...
         prev_cond_expectations] = ...
    kalman_recursion_fast(datamatrix, mu, K, Sigma, A, B, H, ...
                     uncon_mean, uncon_vcv, tolerance)
% KALMAN_RECURSION_FAST constructs the entire Kalman recursion.

%
% Greg Duffee
% March 2005, revised October 2005, Jan 2006, June 2006, August
% 2006, September 2006
%

[T d] = size(datamatrix); 
numfactors = length(mu); 

%
% loop through all observations.
%
errors = zeros(T, d); 
loglikes = zeros(T, 1); 
filtered_values = zeros(T, numfactors); 
prev_cond_expectations = zeros(T,numfactors); 
cond_vars = zeros(T, (numfactors*(numfactors+1)/2));
contemp_vars = zeros(T, (numfactors*(numfactors+1)/2)); 
observed_stepahead_vars = zeros(T, (d+1)*d/2);

Q = Sigma*Sigma'; 
cond_expect = uncon_mean;
cond_var = uncon_vcv; 

%
% Do each period of the recursion separately until the conditional
% variances are sufficiently close.  Then invert the matrices one
% last time, and run through the remaining observations.

index = 1;
old_cond_var = vech(cond_var); 
not_same = 1; 
while ((index <= T) & (not_same == 1))
  prev_cond_expectations(index,:) = cond_expect'; 
  observed_data = datamatrix(index,:)';
  [error loglike contemp_expect contemp_var ...
   forecast_expect forecast_var F] = ...
      single_recursion_noderiv(observed_data, cond_expect, cond_var, ...
                               A, B, H, mu, K, Q);
  loglikes(index) = loglike;
  errors(index,:) = error';
  filtered_values(index,:) = contemp_expect';
  cond_expect = forecast_expect;
  cond_var = forecast_var; 
  cond_vars(index,:) = vech(cond_var)';
  contemp_vars(index,:) = vech(contemp_var)';
  observed_stepahead_vars(index,:) = vech(F)'; 
  index = index + 1; 
  if (sum(abs(vech(cond_var) - old_cond_var)) < tolerance)
    not_same = 0; 
  else
    old_cond_var = vech(cond_var);
  end
end
%
% loop through all other observations, using the last conditional
% variance from above. 
%
if (T >= index) 
  F = B*cond_var*B' + H; 
  detval = log(abs(det(F))); 
  [invF okinv] = invpd(F); 
  if (okinv == 0) 
    loglikes(index + 1) = -1e10; 
  else
    for k = index:T
      prev_cond_expectations(k,:) = cond_expect'; 
      y = datamatrix(k,:)';
      predicted_y = A + B*cond_expect;
      error = y - predicted_y; 
      loglike = -0.5*(detval + error' * invF * error);
      info_in_errors = invF * error; 
      contemp_expect = cond_expect + cond_var * B' * info_in_errors;
      cond_expect = mu + K*contemp_expect;
      loglikes(k) = loglike; 
      errors(k,:) = error'; 
      filtered_values(k,:) = contemp_expect'; 
      cond_vars(k,:) = vech(cond_var)';
      contemp_vars(k,:) = vech(contemp_var)';
      observed_stepahead_vars(k,:) = vech(F)'; 
    end
  end
end

return
