function [errors loglikes ...
          filtered_values cond_vars contemp_vars] = ...
    kalman_recursion_noderiv(datamatrix, mu, K, Sigma, A, B, H, ...
                     uncon_mean, uncon_vcv)
% KALMAN_RECURSION_NODERIV constructs the entire Kalman recursion.

%
% Greg Duffee
% March 2005, revised October 2005, Jan 2006, June 2006, August
% 2006, September 2006
%

[T d] = size(datamatrix); 
numfactors = length(mu); 

%
% loop through all observations.
%
errors = zeros(T, d); 
loglikes = zeros(T, 1); 
filtered_values = zeros(T, numfactors); 
cond_vars = zeros(T, (numfactors*(numfactors+1)/2));
contemp_vars = zeros(T, (numfactors*(numfactors+1)/2)); 
  
Q = Sigma*Sigma'; 
cond_expect = uncon_mean;
cond_var = uncon_vcv; 

for index = 1:T
  observed_data = datamatrix(index,:)';
  [error loglike contemp_expect contemp_var ...
   forecast_expect forecast_var] = ...
      single_recursion_noderiv(observed_data, cond_expect, cond_var, ...
                       A, B, H, mu, K, Q);
  loglikes(index) = loglike;
  errors(index,:) = error';
  filtered_values(index,:) = contemp_expect';
  cond_expect = forecast_expect;
  cond_var = forecast_var; 
  cond_vars(index,:) = vech(cond_var)';
  contemp_vars(index,:) = vech(contemp_var)';
end

return


