function [macrodata yielddata maturities] = read_data(macrodatafile, ...
                          yielddatafile, maturitydatafile, numbonds, ...
                          firstobs, lastobs)
%
% READ_DATA is a routine that reads the data for the estimation problem. 

% Greg Duffee
% October 2005, revised Jan 2006 to focus on single lag case. 
%

%
% the macro data
%

fid = fopen(macrodatafile, 'r');
output = textscan(fid, '%f %f %f', -1); 
macrodata = [output{1} output{2} output{3}]; 
fclose(fid); 

%
% the yields 
% 

formatstring = '%f';
for i = 2:numbonds
    formatstring = [formatstring ' %f'];
end
fid = fopen(yielddatafile, 'r');
output = textscan(fid, formatstring, -1); 
yielddata = []; 
for i = 1:numbonds
    yielddata = [yielddata output{i}];
end
fclose(fid); 

%
% maturities  
% 

fid = fopen(maturitydatafile, 'r');
output = textscan(fid, '%f', -1); 
maturities = output{1};
fclose(fid); 

%
% check on dimensions
%

macro_dim = size(macrodata);
yield_dim = size(yielddata);
maturity_dim = size(maturities); 

if (macro_dim(1) ~= yield_dim(1)) 
     fprintf(1, '\n%s\n', ...
   'WARNING: Macro and yield files do not have the same number of rows.'); 
     fprintf(1, '%s\n\n', ...
   'No failure will occur, but this cannot be good...'); 
end

if (maturity_dim(1) ~= yield_dim(2)) 
     fprintf(1, '\n%s\n', ...
   'WARNING: Number of maturities not equal to number of columns of yields.'); 
     fprintf(1, '%s\n\n', 'Something will soon crash...'); 
end

max_rows = max(macro_dim(1), yield_dim(1)); 
min_rows = min(macro_dim(1), yield_dim(1)); 
used_firstobs = firstobs; 
if (firstobs > min_rows) 
     fprintf(1, '\n%s\n', ...
   'WARNING: Requested first observation exceeds number of rows of');
     fprintf(1, '%s\n', ...
   'either macro or yield files.  Setting first observation to one.');
     used_firstobs = 1; 
end
used_lastobs = lastobs; 
if (lastobs > min_rows) 
     fprintf(1, '\n%s\n', ...
   'WARNING: Requested last observation exceeds number of rows of');
     fprintf(1, '%s\n\n', ...
   'either macro or yield files.  Setting last observation to max allowed.');
     used_lastobs = min_rows; 
end

if (used_firstobs > used_lastobs) 
    fprintf(1, '\n%s\n', ...
   'WARNING: Requested first obs > requested last obs.  Resetting them to');
    fprintf(1, '%s\n\n', 'one and max-allowed, respectively.');
    used_firstobs = 1; 
    used_lastobs = min_rows; 
end

macrodata = macrodata(used_firstobs:used_lastobs,:); 
yielddata = yielddata(used_firstobs:used_lastobs,:); 

end

