function [macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs shortrate_index ...
          use_ols_start use_latent numloops numsimplex numderiv ...
          fund_K_names fund_K_flag fund_K_initialval fund_K_rand_stddev ...
          fund_S_names fund_S_flag fund_S_initialval fund_S_rand_stddev ...
          prem_K_names prem_K_flag prem_K_initialval prem_K_rand_stddev ...
          prem_S_names prem_S_flag prem_S_initialval prem_S_rand_stddev ...
          prem_c_names prem_c_flag prem_c_initialval prem_c_rand_stddev ...
          prem_macro_names prem_macro_flag ...
          prem_macro_initialval prem_macro_rand_stddev ...
          errorSD_names errorSD_flag errorSD_initialval errorSD_rand_stddev ...
          ] = read_input(inputfile, n)
%
% READ_INPUT(inputfile) is a routine that reads the setup of the
% estimation problem from the file 'inputfile'.

% Greg Duffee
% September 2005, revised Jan 2006, September 2006, Jan 2007

%
% open the input file
%

fid = fopen(inputfile, 'r');

%
% read the name of the file with the macro variables
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 9) & (tmp(1:10) == 'MACRO DATA'))
        foundline = 1; 
    end
end
output = textscan(fid, '%s', 1); 
macrodatafile = char(output{1});

%
% read the name of the file with the yields
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 14) & (tmp(1:15) == 'BOND YIELD DATA'))
        foundline = 1; 
    end
end
output = textscan(fid, '%s', 1); 
yielddatafile = char(output{1});

%
% read the name of the file with the maturities 
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'MATURITY DATA'))
        foundline = 1; 
    end
end
output = textscan(fid, '%s', 1); 
maturitydatafile = char(output{1});

%
% read the number of multiperiod bonds 
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'NUMBER OF MUL'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
numbonds = output{1} ;

%
% read the index of the short rate
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'INDEX OF SHOR'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
shortrate_index = output{1} ;

%
% read the length, in years, of a period
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'LENGTH, IN YE'))
        foundline = 1; 
    end
end
output = textscan(fid, '%f', 1); 
periodlength = output{1};

%
% read the index of the first used observation
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'INDEX OF FIRS'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
firstobs = output{1};

%
% read the index of the last used observation
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'INDEX OF LAST'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
lastobs = output{1};

%
% read the flag for using OLS starting values 
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'USE OLS START'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
use_ols_start = output{1};

%
% read the flag for using latent factors 
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'USE LATENT FA'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
use_latent = output{1};

%
% read the number of loops used in numerical estimation
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'NUMBER OF LOO'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
numloops = output{1};

%
% read the number of simplex calls per loop
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'NUMBER OF SIM'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
numsimplex = output{1};

%
% read the number of derivative-based min calls per loop
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 12) & (tmp(1:13) == 'NUMBER OF DER'))
        foundline = 1; 
    end
end
output = textscan(fid, '%d', 1); 
numderiv = output{1};

%
% READING INFORMATION ABOUT PARAMETERS

%
% fundamental var feedback matrix
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 23) & (tmp(1:24) == 'FUNDAMENTAL VAR K MATRIX'))
        foundline = 1; 
    end
end
output = textscan(fid, '%s %d %f %f', n^2); 
fund_K_names = char(output{1}); 
fund_K_flag = output{2};
fund_K_initialval = output{3}; 
fund_K_rand_stddev = output{4};

%
% fundamental var sigma matrix
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 23) & (tmp(1:24) == 'FUNDAMENTAL VAR SIGMA MA'))
        foundline = 1; 
    end
end
output = textscan(fid, '%s %d %f %f', (n*(n+1)/2)); 
fund_S_names = char(output{1}); 
fund_S_flag = output{2};
fund_S_initialval = output{3}; 
fund_S_rand_stddev = output{4};

%
% risk premia var feedback matrix
%

frewind(fid); 
foundline = 0; 
while foundline == 0
  output = textscan(fid, '%[^\n]', 1); 
  tmp = char(output{1}); 
  if ((length(tmp) > 23) & (tmp(1:24) == 'RISK PREMIA VAR K MATRIX'))
    foundline = 1; 
  end
end
output = textscan(fid, '%s %d %f %f', n^2); 
prem_K_names = char(output{1}); 
prem_K_flag = output{2};
prem_K_initialval = output{3}; 
prem_K_rand_stddev = output{4};
  
%
% risk premia var sigma matrix
%

frewind(fid); 
foundline = 0; 
while foundline == 0
  output = textscan(fid, '%[^\n]', 1); 
  tmp = char(output{1}); 
  if ((length(tmp) > 23) & (tmp(1:24) == 'RISK PREMIA VAR SIGMA MA'))
    foundline = 1; 
  end
end
output = textscan(fid, '%s %d %f %f', (n*(n+1)/2)); 
prem_S_names = char(output{1}); 
prem_S_flag = output{2};
prem_S_initialval = output{3}; 
prem_S_rand_stddev = output{4};

%
% risk premia constant terms
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 23) & (tmp(1:24) == 'RISK PREMIA CONSTANT TER'))
        foundline = 1; 
    end
end
output = textscan(fid, '%s %d %f %f', n); 
prem_c_names = char(output{1}); 
prem_c_flag = output{2};
prem_c_initialval = output{3}; 
prem_c_rand_stddev = output{4};

%
% effect of macro vars on risk premia
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 23) & (tmp(1:24) == 'RISK PREMIA EFFECT OF FU'))
        foundline = 1; 
    end
end
output = textscan(fid, '%s %d %f %f', n^2); 
prem_macro_names = char(output{1}); 
prem_macro_flag = output{2};
prem_macro_initialval = output{3}; 
prem_macro_rand_stddev = output{4};

%
% Standard devs of measurement error
%

frewind(fid); 
foundline = 0; 
while foundline == 0
    output = textscan(fid, '%[^\n]', 1); 
    tmp = char(output{1}); 
    if ((length(tmp) > 22) & (tmp(1:23) == 'MEASUREMENT ERROR SIGMA'))
        foundline = 1; 
    end
end
numlines = double(1 + n);
output = textscan(fid, '%s %d %f %f', numlines); 
errorSD_names = char(output{1}); 
errorSD_flag = output{2};
errorSD_initialval = abs(output{3}); 
errorSD_rand_stddev = output{4};

fclose(fid); 

end
