% Greg Duffee
% April 2006, last revision September 2006
%
% Execute this code to recover from an early termination of the
% code used to calculate the ML solution to the term structure
% problem.  The recovery here is useful only if more than one set
% of starting values was used to determine the ML solution. With 
% more than one set, there is a current "best" set of parameters
% that achieve the highest value of the likelihood function.  
% If 'writetemp' == 1, the current best parameters are put into a 
% temporary output file.  The optimization routine can then be 
% restarted from where the routine crashed.  (You may want to
% increment the restart index by one, if the crash was a numerical
% error).  Note that the restart is unaware of the old, best set
% of parameters.  Therefore the code below reads the current best
% value and writes an output file that looks identical to the
% typical output file, but using the temporary "best" values.  This
% output file is called 'tempoutput.dat' in the code below.  
% You must manually compare the ML value in 'tempoutput.dat' with
% the ML value in the normally-named output file.  The ML value in
% 'tempoutput.dat' may be higher, which means it contains the true
% ML solution.  
%
%---------------------------------------------------------------

clear
control_file = char('..\matlabFiles\control.dat'); 

[inputfile old_outputfile] = read_control(control_file); 
temp_outputfile = char('..\matlabFiles\tempoutput.dat'); 
restart_index = crash_recovery(inputfile, temp_outputfile); 

% now restart the ML estimation

printscreen = 1; % print some intermediate results to the screen
writetemp = 1;   % save results in temp files in case of crash 
mineigen_allowed = -1; % keep model away from region where mean
                       % term structure is explosively
                       % zig-zagging.  
kalman_tolerance = 1e-8; % used for fast Kalman recursion, when
                         % estimating with simplex.
restrict_unconmean = 1; % sets likelihood to bad value if
                        % unconditional shape of the term structure
                        % is highly implausible -- either in level
                        % or slope. 
display_simplex = 0; % display iterations in simplex?  1 = yes.
display_deriv = 0; %display iterations in deriv method? 1 = yes. 

[numbonds all_estimates loglike score exitflag ...
 mean_hessian S_matrix T] = ...
    estimate_model(control_file, printscreen, ...
                   writetemp, mineigen_allowed, kalman_tolerance, ...
                   restrict_unconmean, ...
                   display_simplex, display_deriv, restart_index);
