function [freeparams bigparamvec free_index] = setup_initparams( ...
                   use_latent, fund_K_vals, fund_S_vals, ...
                   prem_K_vals, prem_S_vals, prem_c_vals, prem_macro_vals, ...
                   errorSD_vals, fund_K_flag, fund_S_flag, ...
                   prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
                   errorSD_flag, macrodata, ols_flag)
%
% SETUP_INITPARAMS gets the initial vectors of free parameters and
% unrestricted parameters.  
%

%
% Greg Duffee
% September 2006
%

[allparams free_index] = ...
                   pack_allparams(use_latent, ...
                   fund_K_vals, fund_S_vals, ...
                   prem_K_vals, prem_S_vals, prem_c_vals, prem_macro_vals, ...
                   errorSD_vals, fund_K_flag, fund_S_flag, ...
                   prem_K_flag, prem_S_flag, prem_c_flag, prem_macro_flag, ...
                   errorSD_flag); 

if (ols_flag == 1)
  % Get the VAR(1) coefficients for the macro variables.
  [mu_f K_f vcv_f Sigma_f] = get_varparams(macrodata, 1);
  n = length(mu_f); 
  % Check if this VAR is stationary
  [covar_matrix stationary] = uncon_vcv(K_f, vcv_f); 
  if (stationary == 1) 
    % If it is stationary, replace the relevant values in
    % 'allparams' with those from the VAR.  (Replacement occurs
    % only if the variable is a free parameter.)  The variables
    % first have to be stacked as vectors.  
    fund_K_vals = vec(K_f); 
    fund_S_vals = [];
    for i = 1:n
      for j = 1:i
        fund_S_vals = [fund_S_vals; Sigma_f(i,j)];
      end
    end
    % create alternative parameter vector, but do not check whether
    % the relevant variables are free parameters.  Only check to
    % see whether term premium factors are included.  
    if (use_latent == 1) 
      paramvec_ols = [fund_K_vals; fund_S_vals; ...
                      prem_K_vals; prem_S_vals; prem_c_vals; ...
                      prem_macro_vals; errorSD_vals]; 
    else
      paramvec_ols = [fund_K_vals; fund_S_vals; ...
                      prem_c_vals; prem_macro_vals; errorSD_vals]; 
    end
    % Pull out all of the variables associated with free parameters.
    freeparams_ols = pop_freeparams(paramvec_ols, free_index);
    % Poke them into the big parameter vector.  
    bigparamvec = poke_freeparams(freeparams_ols, allparams, free_index); 
  else
    bigparamvec = allparams;
  end
else
  bigparamvec = allparams;
end

freeparams = pop_freeparams(bigparamvec, free_index); 

end
