function [c T R A B chol_H uncon_mean chol_initial_var ...
          shortrate_index periodlength] = ...
            setup_sim(inputfile, sim_maturities, shutoff_latent)
%
% SETUP_SIM returns the vectors and matrices necessary to simulate the
% term structure model.
%

%
% Greg Duffee
% November 2005, revised April 2007
%

n = 3; 

% read the input file to get the relevant information (much of this
% info is irrelevant) 

[macrodatafile yielddatafile maturitydatafile numbonds ...
 periodlength firstobs lastobs shortrate_index ...
 use_ols_start use_latent numloops numsimplex numderiv ...
 fund_K_names fund_K_flag fund_K_vals fund_K_rand_stddev ...
 fund_S_names fund_S_flag fund_S_vals fund_S_rand_stddev ...
 prem_K_names prem_K_flag prem_K_vals prem_K_rand_stddev ...
 prem_S_names prem_S_flag prem_S_vals prem_S_rand_stddev ...
 prem_c_names prem_c_flag prem_c_vals prem_c_rand_stddev ...
 prem_macro_names prem_macro_flag prem_macro_vals prem_macro_rand_stddev ...
 errorSD_names errorSD_flag errorSD_vals errorSD_rand_stddev ...
] = read_input(inputfile, n); 

[macrodata yielddata maturities] = read_data(macrodatafile, ...
          yielddatafile, maturitydatafile, numbonds, firstobs, ...
          lastobs); 

if (shutoff_latent == 1) 
  use_latent = 0;
end

sim_numbonds = length(sim_maturities);

if (use_latent == 1) 
  allparams = [fund_K_vals; fund_S_vals; prem_K_vals; ...
              prem_S_vals; prem_c_vals; prem_macro_vals; errorSD_vals]; 
  
else
  allparams = [fund_K_vals; fund_S_vals; prem_c_vals; ...
              prem_macro_vals; errorSD_vals]; 
  
end

[fund_K fund_S prem_K prem_S prem_c prem_macro errorSD] ...
              = unpack_allparams(n, use_latent, allparams, sim_numbonds); 

[K_f Sigma_f K_omega Sigma_omega lambda0 lambda_1f] = ...
          build_matrices(n, use_latent, ...
                         fund_K, fund_S, prem_K, prem_S, ...
                         prem_c, prem_macro); 
macroMean = mean(macrodata)'; 
mu_f = (eye(n) - K_f) * macroMean;

[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 

cond_var_x = Sigma_x * Sigma_x';
[initial_var initial_valid] = uncon_vcv(K_x, cond_var_x);
chol_initial_var = chol(initial_var)'; 
uncon_mean = inv(eye(size(K_x,1)) - K_x) * mu_x; 

[Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                mu_x_equiv, K_x_equiv, Sigma_x, ...
                                sim_maturities, periodlength); 
[A B H] = measurement_equation(use_latent, Avec, Bmat, errorSD); 

chol_H = diag(sqrt(diag(H))); 

c = mu_x;
T = K_x;
R = Sigma_x;

return

