function [stacked_simple_results, insamp_simple_test_info, ...
          out_of_samp_simple_summary, outsamp_simple_stat_info, ...
          stacked_log_results, insamp_log_test_info, ...
          out_of_samp_log_summary, outsamp_log_stat_info] = ...
    summarize_reg_sims(horizon_index, num_explanatory_vars, ...
                       simple_insamp, simple_outsamp, ...
                       log_insamp, log_outsamp, simple_ma_lag_matrix, ...
                       log_ma_lag_vector)
%
% Greg Duffee
% April 2006
% 
% Summarize results of simulations of forecasting regressions.
% The regressions may be univariate or multivariate, and include
% both in-sample and out-of-sample regressions.  
%

numsims = size(simple_insamp,1);
num_simple_ma_lags = size(simple_ma_lag_matrix,2); 
num_log_ma_lags = length(log_ma_lag_vector); 

side_dim = 1+num_explanatory_vars; 
tot_simple_dim = (1+num_simple_ma_lags)*side_dim + 2*num_simple_ma_lags; 
r_matrix = [zeros(num_explanatory_vars,1) eye(num_explanatory_vars)];

begin_index = tot_simple_dim*(horizon_index - 1) + 1; 
start_point = begin_index; 
these_coefs = simple_insamp(:,start_point:(start_point+num_explanatory_vars));
start_point = begin_index + side_dim; 
these_stderrs = simple_insamp(:,start_point: ...
                             (start_point+num_simple_ma_lags*side_dim-1));
start_point = begin_index + (1+num_simple_ma_lags)*side_dim; 
these_teststats = simple_insamp(:,start_point:(start_point+ ...
    num_simple_ma_lags - 1));
start_point = begin_index + (1+num_simple_ma_lags)*side_dim + ...
    num_simple_ma_lags; 
these_pvals = simple_insamp(:,start_point:(start_point+ ...
    num_simple_ma_lags - 1));

mean_simple_coefs = mean(these_coefs); 
mean_std_errs = reshape(mean(these_stderrs), [], num_simple_ma_lags)'; 
coef_std_devs = sqrt(var(these_coefs, 1)); 
stacked_simple_results = [mean_simple_coefs; coef_std_devs; ...
                    mean_std_errs]; 
simple_reject_rates = [];
for i = 1:num_simple_ma_lags
  simple_reject_rates = [simple_reject_rates; ...
                      sum(these_pvals(:,i) < 0.05)/numsims]; 
end
simple_critvals = [];
for i = 1:num_simple_ma_lags
  sorted = sort(these_teststats(:,i)); 
  simple_critvals = [simple_critvals; sorted(round(0.95*numsims))];
end
insamp_simple_test_info = [simple_reject_rates simple_critvals ...
                   1 - chi2cdf(simple_critvals, num_explanatory_vars)];

% properties of the out of sample ENC regression test.  This is a one-sided
% test.  

if (size(simple_outsamp,1) > 0)
  these_stats = simple_outsamp(:,1:(1+num_simple_ma_lags-1)); 
  outofsamp_critvals = [];
  for i = 1:num_simple_ma_lags
    sorted = sort(these_stats(:,i)); 
    outofsamp_critvals = [outofsamp_critvals; sorted(round(0.95*numsims))];
  end
  critval_5 = norminv(0.95, 0, 1); 
  outofsamp_reject_rates = [];
  for i = 1:num_simple_ma_lags
    outofsamp_reject_rates = [outofsamp_reject_rates; ...
                        sum(these_stats(:,i) > critval_5)/numsims]; 
  end
  out_of_samp_simple_summary = [mean(these_stats); sqrt(var(these_stats, ...
                                                    1))];
  outsamp_simple_stat_info = [outofsamp_reject_rates ...
                      outofsamp_critvals];
else
  out_of_samp_simple_summary = [];
  outsamp_simple_stat_info = [];
end
  
%
% annual log excess returns
%


start_point = 1; 
begin_index = 1; 
these_coefs = log_insamp(:,start_point:(start_point+num_explanatory_vars));
start_point = begin_index + side_dim; 
these_stderrs = log_insamp(:,start_point: ...
                             (start_point+num_log_ma_lags*side_dim-1));
start_point = begin_index + (1+num_log_ma_lags)*side_dim; 
these_teststats = log_insamp(:,start_point:(start_point+ ...
                                                  num_log_ma_lags - 1));
start_point = begin_index + (1+num_log_ma_lags)*side_dim + num_log_ma_lags; 
these_pvals = log_insamp(:,start_point:(start_point+ ...
                                                  num_log_ma_lags - 1));

mean_log_coefs = mean(these_coefs); 
mean_std_errs = reshape(mean(these_stderrs), [], num_log_ma_lags)'; 
coef_std_devs = sqrt(var(these_coefs, 1)); 
stacked_log_results = [mean_log_coefs; coef_std_devs; ...
                    mean_std_errs]; 
log_reject_rates = [];
for i = 1:num_log_ma_lags
  log_reject_rates = [log_reject_rates; sum(these_pvals(:,i) < 0.05)/numsims]; 
end
log_critvals = [];
for i = 1:num_log_ma_lags
  sorted = sort(these_teststats(:,i)); 
  log_critvals = [log_critvals; sorted(round(0.95*numsims))];
end
insamp_log_test_info = [log_reject_rates log_critvals]; 

if (size(log_outsamp,1) > 0)
  these_stats = log_outsamp(:,1:(1+num_log_ma_lags-1)); 
  outofsamp_critvals = [];
  for i = 1:num_log_ma_lags
    sorted = sort(these_stats(:,i)); 
    outofsamp_critvals = [outofsamp_critvals; sorted(round(0.95*numsims))];
  end
  critval_5 = norminv(0.95, 0, 1); 
  outofsamp_reject_rates = [];
  for i = 1:num_log_ma_lags
    outofsamp_reject_rates = [outofsamp_reject_rates; ...
                        sum(these_stats(:,i) > critval_5)/numsims]; 
  end
  outsamp_log_stat_info = [outofsamp_reject_rates ...
                      outofsamp_critvals];
  out_of_samp_log_summary = [mean(these_stats); sqrt(var(these_stats, ...
                                                    1))];
else
  outsamp_log_stat_info = [];
  out_of_samp_log_summary = [];
end

return
