function [meanExcessRets stdExcessRets stdCondMean stdInnov ...
          macroComponent covarComponent residComponent ar1] = ...
    table5_newcode(mu_f, K_f, Sigma_f, K_omega, Sigma_omega, lambda0, ...
                lambda_1f, use_latent, maturities, periodlength, ...
                shortrate_index, filteredFactors)
%
% Greg Duffee
% June 2007
% 
% Rows of table 5.
%

[mu_x K_x Sigma_x mu_x_equiv K_x_equiv] = ...
       build_x_params(use_latent, ...
                      mu_f, K_f, K_omega, Sigma_f, Sigma_omega, ...
                      lambda_1f, lambda0); 
[Avec Bmat] = get_yield_mapping(shortrate_index, ...
                                mu_x_equiv, K_x_equiv, Sigma_x, ...
                                maturities, periodlength); 
delta_x = zeros(size(K_x,1),1);
delta_x(shortrate_index) = 1;
shortmats = maturities - ones(size(maturities, 1), 1);
[shortAvec shortBmat B_pricing_mat] = ...
    get_yield_mapping(shortrate_index, ...
                      mu_x_equiv, K_x_equiv, Sigma_x, ...
                      shortmats, periodlength); 
if (use_latent == 1)  
  lambda1 = [lambda_1f eye(3); zeros(3,6)]; 
  big_lambda0 = [lambda0; zeros(3,1)];
else
  lambda1 = lambda_1f;
  big_lambda0 = lambda0; 
end
scale_matrix = B_pricing_mat*lambda1; 

futureFactors = filteredFactors(2:end,:);
currentFactors = filteredFactors(1:(end-1),:); 
varFactors = cov(filteredFactors,1);

impliedFutureYields = repmat(shortAvec', size(futureFactors,1),1) + ...
    futureFactors *shortBmat';
impliedCurrentYields = repmat(Avec', size(currentFactors,1),1) + ...
    currentFactors*Bmat';
shortReturn = periodlength*currentFactors(:,shortrate_index);
impliedReturns = [];
for i = 1:length(maturities)
  thisRet = periodlength*(maturities(i)*impliedCurrentYields(:,i) - ...
            shortmats(i)*impliedFutureYields(:,i)) - shortReturn;
  impliedReturns = [impliedReturns thisRet];
end
meanExcessRets = mean(impliedReturns);
stdExcessRets = sqrt(var(impliedReturns,1));
stdCondMean = sqrt(diag(scale_matrix * varFactors * scale_matrix')); 
stdInnov = sqrt(diag(B_pricing_mat * Sigma_x*Sigma_x' * ...
                     B_pricing_mat'));

if use_latent
  v11 = varFactors(1:3,1:3);
  v12 = varFactors(1:3,4:6);
  v21 = varFactors(4:6,1:3); 
  v22 = varFactors(4:6,4:6);
  z1 = scale_matrix(:,1:3);
  z2 = scale_matrix(:,4:6);
  macroComponent = z1*v11*z1';
  covarComponent = z2*v21*z1' + z1*v12*z2';
  residComponent = z2*v22*z2';
else
  macroComponent = scale_matrix*varFactors*scale_matrix';
  covarComponent = 0;
  residComponent = 0; 
end

%
% AR(1) coefficient for expected excess returns
%

ar1 = diag((scale_matrix * K_x * varFactors * scale_matrix')) ./ ...
                diag(scale_matrix * varFactors * scale_matrix');


return
