function [covar_matrix valid] = uncon_vcv(F, Q)
% UNCON_VCV constructs the unconditional covariance for x_t where
% x_t = F x_{t-1} + e_t, var(e_t) = Q.  The math follows
% Hamilton, page 265.  'valid' is set to zero if the
% unconditional covariance matrix implies nonstationary dynamics.  

%
% Greg Duffee
% March 2005
%

covar_matrix = []; 
valid = 1;

d = eig(F); 
if (max( d .* conj(d)) >= 1)
    valid = 0; 
else
    vec_sigma = inv( eye(prod(size(F))) - kron(F,F) ) * vec(Q);
    covar_matrix = vec_inverse(vec_sigma);
end


end

