function write_output(inputfile, outputfile, n, ... 
                      fund_K_vals, fund_S_vals, prem_K_vals, ...
                      prem_S_vals, prem_c_vals, prem_macro_vals, ...
                      errorSD_vals, loglike, exitflag, derivatives, ...
                      robust_stderrs, outerprod_stderrs)
%
% WRITE_OUTPUT is a routine that writes the results of estimatng 
% a dynamic term structure model.  The results are written to 'outputfile'.

% Greg Duffee
% October 2005
%

% make an initial call to read_input to get some parameters used
% for writing output.

[macrodatafile yielddatafile maturitydatafile numbonds ...
          periodlength firstobs lastobs ] ...
          = read_input(inputfile, n); 

%
% open the input and output files
%

f_input_id = fopen(inputfile, 'r');
f_output_id = fopen(outputfile, 'w'); 

%
% write the header
%

my_string = '% Output file for estimation of dynamic term structure model'; 
fprintf(f_output_id, '%s\n', my_string); 
my_string = ['% ' datestr(now)]; 
fprintf(f_output_id, '%s\n', my_string); 

%
% copy input file until parameters are reached.  

inputline = textscan(f_input_id, '%[^\n]', 1, 'headerLines', 2); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

foundline = 0; 
while foundline == 0
    inputline = textscan(f_input_id, '%[^\n]', 1); 
    tmp = char(inputline{1}); 
    fprintf(f_output_id, '%s\n', tmp); 
    if ((length(tmp) > 23) & (tmp(1:24) == 'FUNDAMENTAL VAR K MATRIX'))
        foundline = 1; 
    end
end

inputlines = textscan(f_input_id, '%s %d %*f %f', double(3*3)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:(3*3)
    fprintf(f_output_id, '%s %5d %+14.6e %+14.6e\n', param_names(i,:), ...
               param_flags(i), fund_K_vals(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', 6); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:6
    fprintf(f_output_id, '%s %5d %+14.6e %+14.6e\n', param_names(i,:), ...
               param_flags(i), fund_S_vals(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', 9); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:9
    fprintf(f_output_id, '%s %5d %+14.6e %+14.6e\n', param_names(i,:), ...
               param_flags(i), prem_K_vals(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', 6); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:6
    fprintf(f_output_id, '%s %5d %+14.6e %+14.6e\n', param_names(i,:), ...
               param_flags(i), prem_S_vals(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', 3); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:3
    fprintf(f_output_id, '%s %5d %+14.6e %+14.6e\n', param_names(i,:), ...
               param_flags(i), prem_c_vals(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

inputlines = textscan(f_input_id, '%s %d %*f %f', 9); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:9
    fprintf(f_output_id, '%s %5d %+14.6e %+14.6e\n', param_names(i,:), ...
               param_flags(i), prem_macro_vals(i), param_stddevs(i)); 
end

inputline = textscan(f_input_id, '%[^\n]', 1); 
fprintf(f_output_id, '%s\n', char(inputline{1})); 

len_index = 1 + 3; 

inputlines = textscan(f_input_id, '%s %d %*f %f', double(len_index)); 
param_names = char(inputlines{1}); 
param_flags = inputlines{2};
param_stddevs = inputlines{3};
for i = 1:len_index
  fprintf(f_output_id, '%s %5d %+14.6e %+14.6e\n', param_names(i,:), ...
           param_flags(i), errorSD_vals(i), param_stddevs(i)); 
end

%
% write the log-likelihood value and exit flag.  
%

my_string = 'LOG-LIKELIHOOD VALUE'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%f\n', loglike); 

my_string = 'OPTIMIZATION EXIT FLAG'; 
fprintf(f_output_id, '%s\n', my_string); 
fprintf(f_output_id, '%d\n', exitflag); 

my_string = 'DERIVATIVES OF LOG-LIKELIHOOD FUNCTION'; 
fprintf(f_output_id, '%s\n', my_string); 
for i = 1:length(derivatives)
  fprintf(f_output_id, '%d %f\n', i, derivatives(i)); 
end

my_string = 'STANDARD ERRORS (Robust, then outer product)'; 
fprintf(f_output_id, '%s\n', my_string); 
for i = 1:length(outerprod_stderrs)
  fprintf(f_output_id, '%d %12.8f %12.8f\n', i, robust_stderrs(i), ...
          outerprod_stderrs(i)); 
end

fclose(f_input_id);
fclose(f_output_id); 

return

